
/*                          MetaDataFileInterface
 
   This class basically deals with the MetaData file. A meta data file is where
   all the details about a source are stored. MetaData file is an XML file 
   describing each source with fields like type, content, last visited etc...
 */



package CMS;

// Importing Apache Xerces related libraries
import org.apache.xerces.xni.parser.XMLDocumentFilter;
import org.apache.xerces.xni.parser.XMLInputSource;
import org.apache.xerces.xni.parser.XMLParserConfiguration;
import org.apache.xerces.parsers.*;

// Importing JAXP & JAVA DOCUMENT related libraries
import java.io.IOException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;


// To write a DOM to a file
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.FileOutputStream;

// Importing JAXEN related libraries
import org.jaxen.dom.DOMXPath;

// Importing DOM related libraries
import org.w3c.dom.*;

// Importing general Java libraries
import java.io.*;


class MetaDataFileInterface
{
   // Declaring instance variables
   String  metaDataFile = "DataSource_Metadata_Modified.xml";
   
   public String getSourceNamesFromFile(String nodeName)
    {
       String sourceNames = "";
       try
       {
       	  // Forming a document object of the sourceMetadata file      
          Document doc = formDocument(metaDataFile);
          NodeList nl1 = doc.getElementsByTagName(nodeName);
          int noNodes = nl1.getLength();
          //System.out.println("Number of nodes..." + noNodes);
          for(int i =0;i<noNodes+1;i++)
          {
          	Node nd = nl1.item(i);
          	NamedNodeMap nmap = nd.getAttributes(); 
          	Node id = nmap.getNamedItem("id"); 
           	sourceNames = sourceNames + id.getNodeValue() + "*"; 
           	//System.out.println("Source name ..." + sourceNames);
          }
       }
       catch(Exception e)	
       {
       	System.out.println("Exception occured in: getNodeListForSource()");
       }
       return(sourceNames);
    }
    
    /* Return details corresponsing to a source.
       The name of the source is passed as the parameter.
       Details are retunred as a string array
    */  
       

    public String[] getSourceDetails(String sourceName)
    {
      //System.out.println("Inside getSourceDetails() + for " + sourceName);
       
       String[] nodeNames = {"type","content","format","lastModifiedOn","lastVisitedOn","freqOfVisits"};
       
       //System.out.println("here 1..........1");
       String[] returnVals = new String[nodeNames.length];
       try
       {
       	  // Forming a document object of the sourceMetadata file      
          Document doc = formDocument(metaDataFile);
          Element srcElt = doc.getElementById(sourceName); // get src's root element
          //System.out.println("" + srcElt.getTagName());
          for(int i=0;i<nodeNames.length;i++)
          {
          	NodeList nl1 = srcElt.getElementsByTagName(nodeNames[i]);//get all category elements within the source
            Node nd1 = nl1.item(0); // Get the first node in the nodeList
            returnVals[i] = nd1.getFirstChild().getNodeValue().trim();
            //System.out.println("Node values are :" + returnVals[i]);
          }
       }
       catch(Exception e)	
       {
       	System.out.println("Exception occured in: getSourceDetails()");
       }
       return(returnVals);
    }

   /*
    This function returns the list of nodes used to parse data from a source.
    Usually it represents the topology of the source.
    For example, if a source is parsed starting from the List of States in a
    country page,then
    <statesListPage> is the firstNode in the list of nodes returned followed by
    <citiesListPage>, <hotelsListPage> and <hotelsDetailsPage> nodes
   */ 
    
    
    public NodeList getNodeListForSource(String srcHomeURL, String category)
    {
       NodeList nl2 = null;         
       try
       {
       	  // Forming a document object of the sourceMetadata file      
          Document doc = formDocument(metaDataFile);
          
          // Getting the Xpath expressions for a given source and given category
          Element srcElt = doc.getElementById(srcHomeURL); // get src's root element
          NodeList nl1 = srcElt.getElementsByTagName(category);//get all category elements within the source
          Node nd1 = nl1.item(0); // Get the first node in the nodeList
                                
           if(nd1.hasChildNodes() == true)
             {
             	Node nd3 = nd1.getFirstChild();
             	while(nd3.getNodeName() != "XPathExpressions")
             	 {
             	 	nd3 = nd3.getNextSibling();
             	 }
             	         
             	if(nd3.getNodeName() == "XPathExpressions")
             	 {
             	 	boolean starter = true;
               	 	if(nd3.hasChildNodes() == true)
             	 	{
             	 		nl2 = nd3.getChildNodes();
             	 	}
             	 }
             }
       }
       catch(Exception e)	
       {
       	System.out.println("Exception occured in: getNodeListForSource()");
       }
       return(nl2);
    }
    
    
   /* This function returns the attributes of a given node within the specified category.
      The result is a NamedNodeMap containg the list of attributes.
      For example, we can get the list of attributes for the hotel category node for a 
      given source
   */    
      
    public NamedNodeMap getCategoryAttributes(String srcHomeURL, String category) 
    {
    	NamedNodeMap nnoMap1 = null;
    	try
    	{
      	 // Forming a document object of the sourceMetadata file      
          Document doc = formDocument(metaDataFile);
          
          // Getting the Xpath expressions for a given source and given category
          Element srcElt = doc.getElementById(srcHomeURL); // get src's root element
          NodeList nl1 = srcElt.getElementsByTagName(category);//get all category elements within the source
          Node nd1 = nl1.item(0); // Get the first node in the nodeList
          nnoMap1 = nd1.getAttributes(); // find the number of attributes in the firstNode
                 // This nnoMap1 variable will be used to form the exact InitialFullURL for a
                   // specific source
         }
         catch(Exception e)
         {
         	System.out.println("Exception occured in: getCategoryAttributes()");
         }
          return(nnoMap1);          
    }
    
    /*This function updates the value of FreqOfVisits node for a given source*/
    
    public  void updateFreqOfVisitsNodeValue(String sourceName,int inc1dec2)
    {
       try
       {
       	  // Forming a document object of the sourceMetadata file      
          Document doc = formDocument(metaDataFile);
          Element srcElt = doc.getElementById(sourceName); // get src's root element
          
          NodeList nl1 = srcElt.getElementsByTagName("freqOfVisits");//get specific element 
          Node nd1 = nl1.item(0); // Get the first node in the nodeList
          if(inc1dec2 == 1)
             nd1.getFirstChild().setNodeValue("00000000T010000");
          else   
             nd1.getFirstChild().setNodeValue("00000000T004500"); 
          //update changes
          updateFileUsingTransformer(doc);
       }
       catch(Exception e)	
       {
       	System.out.println("Exception occured in: updateFreqOfVisitsNodeValue()");
       }
    }
    
    /*
       This function updates the node value of a specific node with the given value
       This function was meant to update lastModified and lastVisited nodes of a 
       given source
     */
    
    public void updateTimeNodeValue(String sourceName,String nodeName,String nodeValue)
    {
       try
       {
       	  // Forming a document object of the sourceMetadata file      
          Document doc = formDocument(metaDataFile);
          Element srcElt = doc.getElementById(sourceName); // get src's root element
          
          NodeList nl1 = srcElt.getElementsByTagName(nodeName);//get specific element 
          Node nd1 = nl1.item(0); // Get the first node in the nodeList
          //System.out.println("Before node value is updated :" + nodeName + ":" + nd1.getFirstChild().getNodeValue());
          nd1.getFirstChild().setNodeValue(nodeValue);
          //System.out.println("After node value is updated :" + nodeName + ":" + nd1.getFirstChild().getNodeValue());
          
          //update changes
          updateFileUsingTransformer(doc);
      }
       catch(Exception e)	
       {
       	System.out.println("Exception occured in: updateTimeNodeValue()");
       }
    }
    
    /*
     This function is used to write the givee Document object into a given file.
     It was meant to update the changes made to the Document object 
     (by updateTimeNodeValue() and updateFreqOfVisitsNodeValue()) by writing it
     back to the SourceMetaDataFile.
     */
    
    public void updateFileUsingTransformer(Document doc)
    {
    	try{
    	DOMSource source = new DOMSource(doc);
        StreamResult result = new StreamResult(new FileOutputStream(metaDataFile));
	    TransformerFactory transFactory = TransformerFactory.newInstance();
        Transformer transformer = transFactory.newTransformer();
        transformer.transform(source, result);	
        appendMissedDTDPart();
        }
        catch(Exception e)
        {
        	System.out.println("Exception occured in updateFileUsingTransformer() of MetaDataFileInterface.");
        }
   }
   
   /*
    This function appends the InternalDTD part of a file that is omitted while
    writing the Document object 'doc' to the file using updateFileUsingTrasformer().
    */
    
   
   public void appendMissedDTDPart()
   {
   	try{
   	    //String  metaDataFile = "DataSource_Metadata_Modified.xml";
        FileInputStream  fistream = null;
        FileOutputStream fostream = null;  
        BufferedReader br = null;
        PrintStream p = null;
         
        fistream = new FileInputStream(metaDataFile);
        fostream = new FileOutputStream("temp.xml");
        br = new BufferedReader(new InputStreamReader(fistream));
        p = new PrintStream(fostream);
         
        while(true)
		{
            String temp = br.readLine();
            
            if(temp!=null)
              {
              	if(temp.toLowerCase().indexOf("<?xml version=\"1.0\" encoding=")!= -1)
            	{	
            	String replacingString = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" + 
            	                         "<!DOCTYPE sourceDescriptions[<!ATTLIST source id ID #REQUIRED>]>";  
                temp = replacingString;
           		}
              	
              	//System.out.println("" + temp);
              	p.println("" + temp);
              }
               
            else 
               break;   
		}
		br.close();
		p.close();
		
		
		fistream = new FileInputStream("temp.xml");
        fostream = new FileOutputStream(metaDataFile);
        br = new BufferedReader(new InputStreamReader(fistream));
        p = new PrintStream(fostream);
        
        while(true)
		{
            String temp = br.readLine();
            
            if(temp!=null)
              {
              	//System.out.println("" + temp);
              	p.println("" + temp);
              }
            else 
               break;   
		}
		br.close();
		p.close();

      }
      catch(Exception e)
      {
      	System.out.println("Exception occured in : appendMissedDTDPart() of MetaDataFileInterface");
      }
   }
    
    
    // function to form an DOCUMENT object given an XML source file
    public Document formDocument(String fileName) throws Exception
    {
	    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
	    DocumentBuilder builder = factory.newDocumentBuilder();
	    Document document = builder.parse(fileName);
	    return(document);
    }
    
}
