/*                              PERSISTENTONTOLOGY
   
   This class provides an interface for the CMS to store data as Jena Models into the 
   database. The database used is MySQL Server.
 */

// Package
package CMS;

// Import general Java libraries
import java.io.*;
import java.util.*;

// Import Jena related libraries
import com.hp.hpl.jena.db.*;
import com.hp.hpl.jena.ontology.*;
import com.hp.hpl.jena.rdf.model.*;


public class PersistentOntology 
{
    //public static final String ONT1 = "urn:x-hp-jena:test1";
    //public static final String ONT2 = "urn:x-hp-jena:test2";
    
    public static final String ONT_CMK = "world";

    public static final String DB_URL = "jdbc:mysql://localhost/cmk_cms";
    public static final String DB_USER = "CMSAGENT";
    public static final String DB_PASSWD = "CMSPASS";
    public static final String DB = "MySQL";
    String HotelNS = "http://www.agentlab.com/schemas/hotel.rdf#";

    /*
     This function is used to read the specified model present in the database
     into a file name passed as an argument
    */

    public void readSchema_DataModel(String schemaName, String opFileName)
   {
   	     FileOutputStream fostream; PrintStream p;
         try{
         	fostream = new FileOutputStream(opFileName);
            p = new PrintStream(fostream);
         	
         	System.out.println("Data model to be read is : " + schemaName);
         	
         	ModelMaker maker = getMaker();
                
        	//open a previously created model
			Model prvModel = maker.openModel(schemaName);
			
			// list the statements in the Model
			StmtIterator iter = prvModel.listStatements();
			
			while (iter.hasNext()) 
			{
			    Statement stmt      = iter.nextStatement();  // get next statement
			    Resource  subject   = stmt.getSubject();     // get the subject
			    Property  predicate = stmt.getPredicate();   // get the predicate
			    RDFNode   object    = stmt.getObject();      // get the object

			    System.out.print(subject.toString()); 
			    
			    System.out.print(" " + predicate.toString() + " ");  
			    p.print(subject.toString()); 
			    p.print(" " + predicate.toString() + " ");  
			    
			    
			    if (object instanceof Resource) 
			    {
			       System.out.print(object.toString());
			       p.print(object.toString());
			    } 
			    else 
			    {
			        // object is a literal
			        System.out.print(" \"" + object.toString() + "\"");
			        p.print(" \"" + object.toString() + "\"");
			    }
			    System.out.println(" .");
			    System.out.println("----------------------------------------------------------------------");
			    
			    p.println(" .");
			    p.println("----------------------------------------------------------------------");
			 }
			 // Close the database connection            
	    	//conn.close();  
	   } 
	   catch(Exception e){System.out.println("There is an exception:" + e);}
   } // read dataModel

    /*
     This function reloads the model into the database. By this, it removes an old model
     with the same name(if one exists) and stores a new copy. The data for the model
     is read from the file name specified. This file name is the actual ontology
    */

    protected OntModel reloadDB() 
    {
        ModelMaker maker = getMaker();
        // clear out the old
        if (maker.hasModel( ONT_CMK )) maker.removeModel( ONT_CMK );


        // create a spec for the new ont model
        OntModelSpec spec = new OntModelSpec( OntModelSpec.RDFS_MEM );
        spec.setModelMaker( maker );

        // create the base model as a persistent model
        Model base = maker.createModel( ONT_CMK );
        OntModel m = ModelFactory.createOntologyModel( spec, base );
          // Use the persistent model created in the DB via ModelMaker as the base model

        // tell m where to find the content for ontology
        //m.getDocumentManager().addAltEntry( ONT_CMK, "file:world.rdf" );

        // now load the document for test1, which will import ONT_CMK
        m.read( "file:world_Original.rdf" );
        return(m);
    }

   /* 
    This function lists the classes available in the model
    */

    protected void listClasses() 
    {
        OntModel m = getCustomOntologyModel();

         for (Iterator i = m.listClasses(); i.hasNext(); ) {
            OntClass c = (OntClass) i.next();
            System.out.println( "Class " + c.getURI() );
         }
    }
    
    /* 
    This function lists the properties specific to a class
    */
    protected void listClassProperties(String NS, String className)
    {
    	OntModel m = getCustomOntologyModel();
    	OntClass hotel = m.getOntClass( NS + className );
    	
    	for (Iterator i = hotel.listDeclaredProperties(); i.hasNext(); ) 
		 {	
		  OntProperty c = (OntProperty) i.next();
		  System.out.print( c.getLocalName() + " " );
		 }
    	
    }
    
    /* 
     This function creates a class instance based on a specified class with the given 
     namespace and name
     */
    
    protected Individual createClassInstance(String NS, String className, String instanceID)
    {
	    OntModel m = getCustomOntologyModel(); 
	    OntClass cl = m.getOntClass( NS + className );
	    Individual inst = m.createIndividual( instanceID,cl);		
	    return(inst);
	}
   	
   	
   	/*
   	 This function adds values to properties of an instance of a class.
   	 It checks whether the given property wit the given value already exists.
   	 If it is true, it simple false. Otherwise, it returns TRUE
   	 denoting that the model was updated. This inturn means there were
   	 new data available from the source
   	 */
   	 
   	protected boolean addPropertyValuesToInstance(Individual inst, String NSpace,String propertyName, String propertyValue )
   	{
   		
   		
   		//System.out.println("INSTANCE: " + inst.getLocalName());
   		//System.out.println("PROPERTY NAME: " + propertyName);
   		//System.out.println("PROPERTY VALUE: " + propertyValue);
   		
   		
   		
   		OntModel m = getCustomOntologyModel();
  		OntProperty p1 = m.getOntProperty(NSpace + propertyName);
  		
  		propertyValue = propertyValue.replaceAll(" ","");
   		//propertyValue = propertyValue.replaceAll("[","");
   		//propertyValue = propertyValue.replaceAll("]","");
   		propertyValue = propertyValue.replaceAll(":","");
   		propertyValue = propertyValue.replaceAll("text","");
   		
   		if(inst.hasProperty(p1,propertyValue) == false)
   		 {
   		   if(propertyName.compareTo("hotelAmenity") == 0)
   		   {  
   		 	try
   		 	{
   		 		//OntResource range = (OntResource) p1.getRange();
   		 		//OntClass rangeCl = (OntClass) range.as(OntClass.class);
   		 		//OntResource instanceValsArr[] = printIndividuals(NSpace,rangeCl.getLocalName());
   		 		//System.out.println("Hotel Amenity value is:==>" + propertyValue);
   		 		inst.addProperty(p1,propertyValue);	
   		 	}
   		 	catch(Exception e1)
   		 	{
   		 		System.out.println("OntProfile exception" + e1);
   		 	}
   		  }
   		  else
   		   {
   		     inst.addProperty(p1,propertyValue);	
   		   }  	
   		  return(true);
   		 }
   		else
   		{
   			System.out.println("A property with the same value already exists");
   			return(false);
   		} 
 	}
   	
   	/*
   	 This function prints the individuals of a class or all the individuals
   	 present in the model
   	 */
   	
   	protected OntResource[] printIndividuals(String NSpace, String className)
   	{
   	    OntResource returnValArr[] = null;	
   	  try
   	  {
   		//System.out.println("Individuals of class..." + className + "are...");
   		OntModel m = getCustomOntologyModel(); 
   		OntClass cl = null;
   		boolean classNameIsNull = true;
   		Iterator itr = null;
   		
   		if(className == "") // Implies list all individuals in the model
   		{
   			itr = m.listIndividuals();
   		}
   		else // List only individuals of the given class name
   		{
   			cl = m.getOntClass( NSpace+className);
   			itr = cl.listInstances();
   			classNameIsNull = false;
   		}
   		
        // count no of individuals and initialize returnValues array accordingly
   		int noIndi = 0;
   		while(itr.hasNext())
   		{
   		    noIndi++;	
   		    itr.next();
   		}
  		returnValArr = new OntResource[noIndi];
  		
  		// reinitialize iterators
   		if(classNameIsNull == false)
   		   itr = cl.listInstances();
   		else
   		   itr = m.listIndividuals();  

        
        // populate the returnValues array
        int index = 0;
   		while(itr.hasNext())
   		{
   			  int tempI = index;
   			  returnValArr[index++] = (OntResource)itr.next();
   		      //System.out.println(""+ returnValArr[tempI]);
   		}
      }
   	  catch(Exception e)
   	  {
   			System.out.println("Exception in printIndividuals of PersistentOntology" + e);
   	  }
   		
   	  // return the results 
   	  return(returnValArr);
  	}
   	
   	/*
   	 It prints the values corresponding to a property for the given
   	 individual
   	 */
   	
   	protected void printPropertyValuesOfIndividuals(Individual ind, OntProperty p)
   	{
   			// Form a statementIterator over the list of properties.
   			// If a null value is passed for p, list all propeies and their values 
   			   // for the specified individual
   			StmtIterator iter = null;   
   			if(p == null)
			  iter = ind.listProperties();  
			else
			  iter = ind.listProperties(p);
			
			while (iter.hasNext()) 
			{
			    Statement stmt      = iter.nextStatement();  // get next statement
			    Resource  subject   = stmt.getSubject();     // get the subject
			    Property  predicate = stmt.getPredicate();   // get the predicate
			    RDFNode   object    = stmt.getObject();      // get the object
			    
			    if (object instanceof Resource) 
			    {
			       System.out.println(object.toString());
			    } 
			    else 
			    {
			        System.out.println(" \"" + object.toString() + "\"");
			        
			    }
			 }
   	}
   	
    /*
     It returns the model object created using custom 
     parameters
     */
    
    protected OntModel getCustomOntologyModel()
    {
      OntModelSpec spec = new OntModelSpec( OntModelSpec.RDFS_MEM );
      spec.setModelMaker( getMaker() );
      OntModel m = ModelFactory.createOntologyModel( spec, getMaker().createModel( ONT_CMK ) );
      return m;	
    }
    
    /*
     Returns an instance of the model maker class used to 
     create a persistent model in the database
     */
    protected ModelMaker getMaker() 
    {
        try {
            // Load the Driver
            String className = "com.mysql.jdbc.Driver";
            Class.forName(className);

            // Create database connection
            IDBConnection conn  = new DBConnection ( DB_URL, DB_USER, DB_PASSWD, DB );

            // Create a model maker object
            return ModelFactory.createModelRDBMaker(conn);
        }
        catch (Exception e) {
            e.printStackTrace();
            System.exit( 1 );
        }

        return null;
    }
}


