/*  
                           *  SOURCE SCHEDULER SCREEN * 
 *  This java class performs the following functionalities:
 *  1. Creates the source scheduler screen by extending the JFrame class
 *  2. Provides facilities for the CMS administrator to schedule an existing source for data gathering
 */

package CMS;

// importing general Java libraries
import java.io.*;
import java.util.*;

// importing Swing and AWT related libraries
import javax.swing.*;
import javax.swing.JComponent;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

// Importing data related Java libraries
import java.util.*;
import java.util.Date;
import java.util.Calendar;
import java.util.GregorianCalendar;


// Importing Quartz Scheduler related libraries
import org.quartz.CronTrigger;
import org.quartz.JobDetail;
import org.quartz.Scheduler;
import org.quartz.SchedulerFactory;
import org.quartz.SchedulerMetaData;
import org.quartz.SimpleTrigger;
import org.quartz.helpers.TriggerUtils;
import org.quartz.JobListener;

import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;




class SourceScheduler extends JFrame implements ActionListener
{
	
	/******************************** 
	 *                              *
	 *Declare All instance variables* 
	 *                              *
	 ********************************/
	
	// Declaring source meta data file
	String srcMetaDataFile = "DataSource_Metadata_Modified.xml";
	
	// Declaring the WEST panel in the frame and the components inside
	JLabel srcListLbl = new JLabel("List of sources in the DB");
	JPanel srcListLblPanel = new JPanel();
	
	JPanel srcsListPanel = new JPanel();
	JScrollPane scrollPane1; // Scrollpane 1 for listing all sources
	JList sourceList = new JList(); 
	DefaultListModel sourceListModel = new DefaultListModel();
	JButton selectSrcBtn = new JButton("Select the highlighted source");
	
	// Declaring the CENTER panel and the components inside
	JPanel srcLabelValuePanel = new JPanel();
	JLabel srcLinkLabel = new JLabel("Source Link - URL Of the source:");// will go as the resource URI
	JTextField srcLinkText = new JTextField();
	JLabel srcTypeLabel = new JLabel("Source Type - HTML Page, RSS Feed, RDF:");
	JTextField srcTypeText = new JTextField();
	JLabel srcContentLabel = new JLabel("Content Type - Hotels, Restaurants, Parks, Others:");
	JTextField srcContentText = new JTextField();
	JLabel srcFormatLabel = new JLabel("Source Format - Structure of the document (N/A for now):");
	JTextField srcFormatText = new JTextField();
	JLabel srcLastModifiedLabel = new JLabel("Source Last Modified on - (YYYYMMDDTHHMinMinSS):");
	JTextField srcLastModifiedText = new JTextField();
	JLabel srcLastVisitLabel = new JLabel("Source Last visited on - (YYYYMMDDTHHMMSS):");
	JTextField srcLastVisitText = new JTextField();
	JLabel srcFreqVisitLabel = new JLabel("Frequency of visits - (CCYYMMDDTHHMinMinSS):");
	JTextField srcFreqVisitText = new JTextField();
	
	//Declaring the SOUTH panel in the frame
	JPanel buttonPanel1 = new JPanel(new GridLayout(1,1)); //start gathering button
	JButton gatherSourcesBtn = new JButton("Start Gathering from known sources");
	
	/********************************************** 
	 *
	 *Construtor: Initialize all the panel elements 
	 *
	 **********************************************/
	 
	 public SourceScheduler()
	 {
	 	//Title for screen
		super("RDF BASED CMS SYSTEM - SOURCE DESCRIPTION SCREEN");
				
		//Setting Frame Layout
		getContentPane().setLayout(new BorderLayout());
		
		// Adding components to srcLableValuePanel and the srcLableValuePanel to the frame
		// CENTER PANE - srcLabelValuePanel
		srcLabelValuePanel.setLayout(new BoxLayout(srcLabelValuePanel,BoxLayout.PAGE_AXIS));
		srcLabelValuePanel.add(srcTypeLabel); 		srcLabelValuePanel.add(srcTypeText);
		srcLabelValuePanel.add(srcContentLabel); 	srcLabelValuePanel.add(srcContentText);
		srcLabelValuePanel.add(srcFormatLabel); 	srcLabelValuePanel.add(srcFormatText);
		srcLabelValuePanel.add(srcLastModifiedLabel); 	srcLabelValuePanel.add(srcLastModifiedText);
		srcLabelValuePanel.add(srcLastVisitLabel); 	srcLabelValuePanel.add(srcLastVisitText);
		srcLabelValuePanel.add(srcFreqVisitLabel);	srcLabelValuePanel.add(srcFreqVisitText);
		srcLabelValuePanel.setPreferredSize(new Dimension(600,500)); 
		srcLabelValuePanel.setVisible(false);
		srcLabelValuePanel.setBorder(BorderFactory.createLineBorder (Color.black, 2));   
		Color clr = new Color(149,186,217);		
	    srcLabelValuePanel.setBackground(clr);
	    srcLabelValuePanel.setFont(new Font("Serif", Font.BOLD,15));
		getContentPane().add(srcLabelValuePanel,BorderLayout.CENTER);
		
		// Adding components to model_srcsListPanel and model_srcsListPanel to the frame
		// WEST PANE - srcsListPanel
		
		srcsListPanel.setLayout(new BoxLayout(srcsListPanel, BoxLayout.PAGE_AXIS));
		
	    setLblProps(srcListLbl,15,srcListLblPanel,400,20);
	    srcsListPanel.add(srcListLblPanel);
	    scrollPane1 = new JScrollPane(sourceList);  
	    scrollPane1.setPreferredSize(new Dimension(400,180)); 
	    sourceList.setSelectedIndex(0);
	    srcsListPanel.add(scrollPane1);
	    setBtnProps(selectSrcBtn,12,this,400,30);
	    selectSrcBtn.setAlignmentX(Component.CENTER_ALIGNMENT);
	    srcsListPanel.add(selectSrcBtn);
	    selectSrcBtn.setEnabled(true);
	    
		getContentPane().add(srcsListPanel,BorderLayout.WEST);
		srcsListPanel.setPreferredSize(new Dimension(400,500)); 
		srcsListPanel.setVisible(true);
		srcsListPanel.setBorder(BorderFactory.createLineBorder (Color.black, 2));   
		
		// Adding the lower panel (button) panel to the frame
        // SOUTH PANE - buttonPanel1
          
        setBtnProps(gatherSourcesBtn,15,this,1000,30);   
        gatherSourcesBtn.setEnabled(false);
        buttonPanel1.add(gatherSourcesBtn); 
        buttonPanel1.setBorder(BorderFactory.createLineBorder (Color.black, 2));   
        getContentPane().add(buttonPanel1,BorderLayout.SOUTH);
        
		//Setting Screen properties
		setSize(1000,600);
		setVisible(true);
		//setResizable(false);
		
		// Adding listener for windows
		addWindowListener (
			new WindowAdapter() {
				 public void windowClosing(WindowEvent e) {
				  	System.exit(0);
				  }	} );
				  
	    // Initializing the source list with source names from the metadata file
	    MetaDataFileInterface mdFI = new MetaDataFileInterface();
	    String sourceNames = mdFI.getSourceNamesFromFile("source");
	    //System.out.println("SOurce:------>" + sourceNames);
	    StringTokenizer sTokr = new StringTokenizer(sourceNames,"*");
	    sourceListModel.removeAllElements();
	    
	    while(sTokr.hasMoreTokens() == true)
	    {
	    	sourceListModel.addElement(sTokr.nextToken().trim());
	    }
	    sourceList.setModel(sourceListModel);
	 }// SourceScheduler constructor
	 
   public void setBtnProps(JButton btn,int fontSize, ActionListener obj, int x, int y)
   {
   	   	Color clr = new Color(16,107,141);		
	   	btn.setFont(new Font("Serif", Font.BOLD,fontSize));
	   	btn.setPreferredSize(new Dimension(x,y)); 
		btn.setBackground(clr);
		btn.addActionListener(obj);
   }
   
   public void setLblProps(JLabel lbl, int fontSize, JPanel lblPnl, int x, int y)
   {
   	    Color clr = new Color(149,186,217);		
   	    lbl.setFont(new Font("Serif", Font.BOLD,fontSize));
   	    lblPnl.setBackground(clr);
   	    lblPnl.setPreferredSize(new Dimension(x,y)); 
   	    lblPnl.add(lbl);
   }
   
  public void actionPerformed(ActionEvent e) 
	{
		 //If the button to select a source from list is clicked//
		 if(e.getSource() == selectSrcBtn)
		 {
		 	handleSelectSource();
		 }
		 
	    //If the button to gather data from a source is clicked//	 
	    if(e.getSource() == gatherSourcesBtn)
	    {
	       handleGatherSource();
	    }
	}
	
   
	
   /********************************************************
				 Event handlers for 
				 *    1. Select source button
				 *    2. Gather from source button
	 ********************************************************/
	
  public void handleSelectSource()
  {
  	 //System.out.println("Inside handleSelectSource()");
	 srcLabelValuePanel.setEnabled(true);
 	 srcLabelValuePanel.setVisible(true);
 	 
     try
     {
        MetaDataFileInterface mdFI = new MetaDataFileInterface();
        String vals[] = mdFI.getSourceDetails("" + sourceList.getSelectedValue());
        	srcTypeText.setText(vals[0]) ;            
        	srcContentText.setText(vals[1]);
      		srcFormatText.setText(vals[2]);
      		srcLastModifiedText.setText(vals[3]);   
      		srcLastVisitText.setText(vals[4]);        
      		srcFreqVisitText.setText(vals[5]);   
     }
     catch(Exception e1) 
     {
      	System.out.println("Exception occured while selecting a source" + e1);
     }
 	  gatherSourcesBtn.setEnabled(true);
  } 
  
 public void handleGatherSource()
 {
 	try
 	{
 		MetaDataFileInterface mdFI = new MetaDataFileInterface();
        String vals[] = mdFI.getSourceDetails("" + sourceList.getSelectedValue());
        // vals[]'s format = {"type","content","format","lastModifiedOn","lastVisitedOn","freqOfVisits"};
        
        /* 
	    Steps to schedule a job using Quartz scheduler
	    1. Create a SCHEDULER
		2. Create the JOBDETAIL for the job to be scheduled
		3. Create the TRIGGER for the job
	    4. Associate JobListener/Trigger Listener with the job and trigger respectively(if needed)
	    5. Schedule the job (using the scheduler)
	    */
	    
	    // Creating a scheduler
	    SchedulerFactory sf = new org.quartz.impl.StdSchedulerFactory();
	    Scheduler sched = sf.getScheduler();
	 	sched.addJobListener(new MyJobListener("Listener :" + sourceList.getSelectedValue()));
	 	
	 	
	 		 	
	 	/*                         MODIFICATION A
	 	 * 
	  	 *
	 	 * Change the name of country (and/or) state and state abbreviation in the job name
	 	 * in the given order
	 	 *
	 	 *
	 	 */
	 	
	 	String jobName = vals[1] + "*" + sourceList.getSelectedValue() + "*" + "United States" +
	 	                 "*" + "us" + "*" + "Oklahoma" + "*" + "ok" + "*";
	 	                  
	 	
	 	// Creating a jobDetail                   
	 	JobDetail job = new JobDetail(jobName, vals[1], PageManipulator_Modified.class);
	 	
        // Creating a Trigger
        //extracting fields from lastVisitedOn field
	 	int yyyy = Integer.parseInt(vals[4].substring(0,4));
	 	int mm   = Integer.parseInt(vals[4].substring(4,6));
	 	int dd   = Integer.parseInt(vals[4].substring(6,8));
	 	int hh   = Integer.parseInt(vals[4].substring(9,11));
	 	int min  = Integer.parseInt(vals[4].substring(11,13));
	 	int ss   = Integer.parseInt(vals[4].substring(13,15));
	 	
	 	// Setting values to the Calendar data type
	 	java.util.Calendar cal = new java.util.GregorianCalendar(yyyy,(mm-1) ,dd ); 
	 	// month starts with January = 0. Feb = 1,...
	  	cal.set(cal.HOUR, hh);
	  	cal.set(cal.MINUTE, min);
	  	cal.set(cal.SECOND, ss);
	  	cal.set(cal.MILLISECOND, 0);
	  	
	  	Date startTime = cal.getTime();
	  	System.out.println("Previously visited on : " + startTime);
	  	
	    // Creating a trigger with the above time values
	    SimpleTrigger trigger = new SimpleTrigger("Trigger :" + sourceList.getSelectedValue(),
	                                    vals[0],
	                                    startTime,
	                                    null,
	                                    0,
	                                    0);
	    
	    // Associating a listener with the job
		//MyJobListener mjl = new MyJobListener("Listener :" + sourceList.getSelectedValue());
		job.addJobListener("Listener :" + sourceList.getSelectedValue());
		
		                                           
	    // Scheduling the job using the scheduler (with jobdetails and trigger)
	    
	    Date ft = sched.scheduleJob(job, trigger);
	    System.out.println("Job name is: " + job.getName());
	    //System.out.println("Ready for scheduling...");
	    sched.start();
	    //System.out.println("Scheduled...");
	}
	catch(Exception e)
	{
		
	}
 }
}

// Defining the class that listens to events associated with a scheduled job//
class MyJobListener implements JobListener 
{
	String name;
       
    public MyJobListener(String name) 
    {
        this.name = name;
    }

    public String getName() 
    {
        return name;
    }

    public void jobToBeExecuted(JobExecutionContext context) 
    {
        System.err.println(".......(" + name + "): jobToBeExecuted: "
                + context.getJobDetail().getFullName());
    }

    public void jobWasExecuted(JobExecutionContext context,JobExecutionException jobException) 
    {
        
        System.err.println(".......(" + name + "): jobWasExecuted: "
         + context.getJobDetail().getFullName());
         
       try
       {
          String s1 = context.getJobDetail().getName();
	      StringTokenizer sTokr = new StringTokenizer(s1,"*");
	      String vals[] = new String[sTokr.countTokens()];
	      int index = 0;
	      while(sTokr.hasMoreTokens())
	      {
	     	vals[index++] = sTokr.nextToken();
	      } 
                
        MetaDataFileInterface mdFI = new MetaDataFileInterface();
        String srcVals[] = mdFI.getSourceDetails("" + vals[1]);
        // srcVals[]'s format = {"type","content","format","lastModifiedOn","lastVisitedOn","freqOfVisits"};        
        
        /* 
         1. Get all the details from the context (like jobname, triggername, groupname and the date.time
            it was scheduled
         2. Calculate the next visit date (using the above details and freqOfVisits data from the DBase
         3. Create a new trigger with the 'next visit date' value
         4. Reschedule the job using the new trigger
        */
                   
         System.out.println("Prev Scheduled Time was : " + context.getTrigger().getPreviousFireTime());
         GregorianCalendar cal1 = new GregorianCalendar();
         cal1.setTime(context.getTrigger().getPreviousFireTime());
         
         int cc1 = Integer.parseInt(srcVals[5].substring(0,2));
	  	 int yy1 = Integer.parseInt(srcVals[5].substring(2,4));
	  	 int mm1 = Integer.parseInt(srcVals[5].substring(4,6)); 
	  	 int dd1 = Integer.parseInt(srcVals[5].substring(6,8)); 
	  	 int hh1 = Integer.parseInt(srcVals[5].substring(9,11));
	  	 int min1 = Integer.parseInt(srcVals[5].substring(11,13));
	  	 int sec1 = Integer.parseInt(srcVals[5].substring(13,15));
	  	 
	  	 // adding the values from freqOfVisits field to the srcLastVisit field
	  	 cal1.add(Calendar.YEAR, yy1);
	     cal1.add(Calendar.MONTH, mm1);
	     cal1.add(Calendar.DATE, dd1);
	     cal1.add(Calendar.HOUR, hh1);
	     cal1.add(Calendar.MINUTE, min1);
	     cal1.add(Calendar.SECOND, sec1);
	     
	     // Create a new trigger
    	 Date startTime = cal1.getTime();
  		 System.out.println("Updated Time is : " + startTime);
  		 System.out.println("Job Name is     : " + context.getJobDetail().getName());
    	 SimpleTrigger trigger = new SimpleTrigger("Trigger :" + context.getJobDetail().getName() ,
                                    srcVals[1],
                                    ""+context.getJobDetail().getName(),
                                    srcVals[1],
                                    startTime,
                                    null,
                                    0,
                                    0);
         // Reschedule the job with the updated date details
		 context.getScheduler().rescheduleJob(context.getTrigger().getName(),context.getTrigger().getGroup(),trigger);    
	 } //try
	 catch(Exception e)
	 {
	 	System.out.println("Exception occured in : jobWasExecuted() of SourceScheduler");
	 }
   }              
    
   public void jobExecutionVetoed(JobExecutionContext context) 
   {
         // TODO Auto-generated method stub
   }

}

